/*
 * Copyright (C) 2021 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.rest.api.v1;

import java.util.List;

import org.openepics.names.rest.beans.FieldName;
import org.openepics.names.rest.beans.element.NameElement;
import org.openepics.names.rest.beans.element.NameElementCommandConfirm;
import org.openepics.names.rest.beans.element.NameElementCommandCreate;
import org.openepics.names.rest.beans.element.NameElementCommandUpdate;
import org.openepics.names.rest.beans.response.Response;
import org.openepics.names.rest.beans.response.ResponseBoolean;
import org.openepics.names.rest.beans.response.ResponseBooleanList;
import org.openepics.names.rest.beans.response.ResponsePageNameElements;
import org.openepics.names.util.ExcelUtil;
import org.springframework.core.io.Resource;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;

import io.swagger.v3.oas.annotations.Hidden;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.media.ArraySchema;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import io.swagger.v3.oas.annotations.tags.Tag;

/**
 * This part of REST API provides names data for Naming application.
 *
 * @author Lars Johansson
 */
@Tag(name        = "Names",
     description = "handle names data for Naming application")
@RequestMapping("/api/v1/names")
public interface INames {

    /*
       NameElementCommandCreate  -       parentSystemStructure, parentDeviceStructure, index, description, comment
       NameElementCommandUpdate  - uuid, parentSystemStructure, parentDeviceStructure, index, description, comment
       NameElementCommandConfirm - uuid,                                                                   comment
       ----------------------------------------------------------------------------------------------------
       converted into NameElementCommand
       ----------------------------------------------------------------------------------------------------
       NameElementCommand        - uuid, parentSystemStructure, parentDeviceStructure, index, description, comment
       NameElement
                     uuid                     (UUID)
                     parentSystemStructure    (UUID)
                     parentDeviceStructure    (UUID)
                     index                    (String)
                     description              (String)
                     comment                  (String)
                     ----------------------------------
                     systemStructure          (String)
                     deviceStructure          (String)
                     name                     (String)
                     status                   (Status)
                     latest                   (Boolean)
                     deleted                  (Boolean)
                     when                     (Date)
                     who                      (String)

       Methods
           create    POST   /names                                - createNames                  (List<NameElementCommandCreate>)
           create    POST   /names/upload                         - createNames                  (MultipartFile)
           ----------------------------------------------------------------------------------------------------
           read      GET    /names                                - readNames                    (Boolean, String, String, String, String, String, String, FieldName, Boolean, Integer, Integer)
           read      GET    /names/download                       - readNamesDownload            (Boolean, String, String, String, String, String, String, FieldName, Boolean, Integer, Integer)
           read      GET    /names/{name}                         - readNames                    (String, FieldName, Boolean, Integer, Integer)
           read      GET    /names/systemStructure/{mnemonicPath} - readNamesSystemStructure     (String, FieldName, Boolean, Integer, Integer)
           read      GET    /names/deviceStructure/{mnemonicPath} - readNamesDeviceStructure     (String, FieldName, Boolean, Integer, Integer)
           read      GET    /names/history/{uuid}                 - readNamesHistory             (String, FieldName, Boolean, Integer, Integer)
           ----------------------------------------------------------------------------------------------------
           read      GET    /names/equivalence/{name}             - equivalenceName              (String)
           read      GET    /names/exists/{name}                  - existsName                   (String)
           read      GET    /names/isLegacy/{name}                - isLegacyName                 (String)
           read      GET    /names/isValidToCreate/{name}         - isValidToCreateName          (String)
           ----------------------------------------------------------------------------------------------------
           read      GET    /names/validateCreate                 - validateNamesCreate          (List<NameElementCommandCreate>)
           read      GET    /names/validateUpdate                 - validateNamesUpdate          (List<NameElementCommandUpdate>)
           read      GET    /names/validateDelete                 - validateNamesDelete          (List<NameElementCommandConfirm>)
           ----------------------------------------------------------------------------------------------------
           update    PUT    /names                                - updateNames                  (List<NameElementCommandUpdate>)
           update    PUT    /names/upload                         - updateNames                  (MultipartFile)
           ----------------------------------------------------------------------------------------------------
           delete    DELETE /names                                - deleteNames                  (List<NameElementCommandConfirm>)
           delete    DELETE /names/upload                         - deleteNames                  (MultipartFile)

       Note
           read      GET    /names/{name} - both name and uuid (name - exact and search, uuid exact)
           /upload and /download endpoints handle Excel files
           authentication/authorization
               3 levels - no, user, administrator
                   no            - read
                   user          - create, update, delete
                 ( administrator )
     */

    public static final String DEFAULT_PAGE            = "0";
    public static final String DEFAULT_PAGE_SIZE       = "100";
    public static final String DEFAULT_SORT_FIELD_WHEN = "WHEN";
    public static final String DEFAULT_SORT_ORDER_ASC  = "true";

    /**
     * Create names by list of name element commands for create.
     * Return list of name elements for created names.
     *
     * @param nameElementCommands list of name element commands for create
     * @return list of created name elements
     */
    @Operation(
            summary     = "Create names by array of name element commands for create",
            description = """
                          Create names by array of name element commands for create.
                          Return array of name elements for created names.

                          Required attributes:
                          - parentSystemStructure (uuid)
                          - description
                          - comment

                          Optional attributes:
                          - parentDeviceStructure (uuid)
                          - index

                          Uuid is created by Naming.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "201",
                    description  = "OK. Return array of name elements for created names.",
                    content = @Content(
                            mediaType = "application/json",
                            array = @ArraySchema(schema = @Schema(implementation = NameElement.class)))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "404",
                    description  = "Not Found. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "409",
                    description  = "Conflict. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @PostMapping(
            produces = {"application/json"},
            consumes = {"application/json"})
    public ResponseEntity<List<NameElement>> createNames(
            @Parameter(
                    in = ParameterIn.DEFAULT,
                    description = "array of name element commands",
                    required = true,
                    content = @Content(
                            mediaType = "application/json",
                            array = @ArraySchema(
                                    schema = @Schema(
                                            implementation = NameElementCommandCreate.class,
                                            requiredProperties = {"parentSystemStructure","description","comment"}))))
            @RequestBody List<NameElementCommandCreate> nameElementCommands);

    /**
     * Create names by upload Excel file.
     * Return Excel file with name elements for created names.
     *
     * @param file file
     * @return Excel file
     */
    @Operation(
            summary     = "Create names by upload Excel file",
            description = """
                          Create names by upload Excel file.
                          Return Excel file with name elements for created names.

                          Expected columns as in Excel template for names.

                          Required attributes:
                          - parentSystemStructure (uuid)
                          - description
                          - comment

                          Optional attributes:
                          - parentDeviceStructure (uuid)
                          - index

                          Uuid is created by Naming.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "201",
                    description  = "OK. Return Excel file with name elements for created names.",
                    content = @Content(mediaType = ExcelUtil.MIME_TYPE_EXCEL)),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "404",
                    description  = "Not Found. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "409",
                    description  = "Conflict. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @PostMapping(
            value = "/upload",
            produces = {"application/vnd.ms-excel"},
            consumes = {"multipart/form-data"})
    public ResponseEntity<Resource> createNames(
            @Parameter(
                    in = ParameterIn.DEFAULT,
                    description = "Excel file with name element commands",
                    required = true,
                    content = @Content(mediaType = ExcelUtil.MIME_TYPE_OPENXML_SPREADSHEET))
            @RequestParam("file") MultipartFile file);

    // ----------------------------------------------------------------------------------------------------

    /**
     * Find valid names (search).
     * Return paged list of name elements.
     *
     * @param deleted if deleted-only names are to be included
     * (false for non-deleted-only names, true for deleted-only names, not used for both cases)
     * @param uuid uuid
     * @param name name
     * @param nameEquivalence name equivalence
     * @param systemStructure system structure mnemonic path
     * @param deviceStructure device structure mnemonic path
     * @param index index
     * @param description description
     * @param orderBy order by field
     * @param isAsc sort order, ascending or descending
     * @param page page starting from 0, offset
     * @param pageSize page size, limit
     * @return paged list of name elements
     */
    @Operation(
            summary     = "Find valid names (search)",
            description = """
                          Find valid names (search).
                          Return paged array of name elements.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return paged array of name elements.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = ResponsePageNameElements.class))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            produces = {"application/json"})
    public ResponsePageNameElements readNames(
            @Parameter(in = ParameterIn.QUERY, description = "if deleted names are to be included or not, omitted for both deleted and not deleted names") @RequestParam(required = false) Boolean deleted,
            @Parameter(in = ParameterIn.QUERY, description = "search by uuid") @RequestParam(required = false) String uuid,
            @Parameter(in = ParameterIn.QUERY, description = "search by name") @RequestParam(required = false) String name,
            @Parameter(in = ParameterIn.QUERY, description = "search by name equivalence") @RequestParam(required = false) String nameEquivalence,
            @Parameter(in = ParameterIn.QUERY, description = "search by system structure mnemonic path") @RequestParam(required = false) String systemStructure,
            @Parameter(in = ParameterIn.QUERY, description = "search by device structure mnemonic path") @RequestParam(required = false) String deviceStructure,
            @Parameter(in = ParameterIn.QUERY, description = "search by index") @RequestParam(required = false) String index,
            @Parameter(in = ParameterIn.QUERY, description = "search by description") @RequestParam(required = false) String description,
            @Parameter(in = ParameterIn.QUERY, description = "order by field") @RequestParam(required = false) FieldName orderBy,
            @Parameter(in = ParameterIn.QUERY, description = "sort order, ascending or descending") @RequestParam(required = false) Boolean isAsc,
            @Parameter(in = ParameterIn.QUERY, description = "page starting from 0, offset") @RequestParam(required = false, defaultValue = DEFAULT_PAGE) Integer page,
            @Parameter(in = ParameterIn.QUERY, description = "page size, limit") @RequestParam(required = false, defaultValue = DEFAULT_PAGE_SIZE) Integer pageSize);

    /**
     * Find valid names (search).
     * Return Excel file with paged list of name elements.
     *
     * @param deleted if deleted-only names are to be included
     * (false for non-deleted-only names, true for deleted-only names, not used for both cases)
     * @param uuid uuid
     * @param name name
     * @param nameEquivalence name equivalence
     * @param systemStructure system structure mnemonic path
     * @param deviceStructure device structure mnemonic path
     * @param index index
     * @param description description
     * @param orderBy order by field
     * @param isAsc sort order, ascending or descending
     * @param page page starting from 0, offset
     * @param pageSize page size, limit
     * @return Excel file
     */
    @Operation(
            summary     = "Find valid names (search) and download Excel file",
            description = """
                          Find valid names (search).
                          Return Excel file with paged list of name elements.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return Excel file with paged list of name elements.",
                    content = @Content(mediaType = ExcelUtil.MIME_TYPE_EXCEL)),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            value = "/download",
            produces = {"application/vnd.ms-excel"})
    public ResponseEntity<Resource> readNamesDownload(
            @Parameter(in = ParameterIn.QUERY, description = "if deleted names are to be included or not, omitted for both deleted and not deleted names") @RequestParam(required = false) Boolean deleted,
            @Parameter(in = ParameterIn.QUERY, description = "search by uuid") @RequestParam(required = false) String uuid,
            @Parameter(in = ParameterIn.QUERY, description = "search by name") @RequestParam(required = false) String name,
            @Parameter(in = ParameterIn.QUERY, description = "search by name equivalence") @RequestParam(required = false) String nameEquivalence,
            @Parameter(in = ParameterIn.QUERY, description = "search by system structure mnemonic path") @RequestParam(required = false) String systemStructure,
            @Parameter(in = ParameterIn.QUERY, description = "search by device structure mnemonic path") @RequestParam(required = false) String deviceStructure,
            @Parameter(in = ParameterIn.QUERY, description = "search by index") @RequestParam(required = false) String index,
            @Parameter(in = ParameterIn.QUERY, description = "search by description") @RequestParam(required = false) String description,
            @Parameter(in = ParameterIn.QUERY, description = "order by field") @RequestParam(required = false) FieldName orderBy,
            @Parameter(in = ParameterIn.QUERY, description = "sort order, ascending or descending") @RequestParam(required = false) Boolean isAsc,
            @Parameter(in = ParameterIn.QUERY, description = "page starting from 0, offset") @RequestParam(required = false, defaultValue = DEFAULT_PAGE) Integer page,
            @Parameter(in = ParameterIn.QUERY, description = "page size, limit") @RequestParam(required = false, defaultValue = DEFAULT_PAGE_SIZE) Integer pageSize);

    /**
     * Find valid names by name or uuid (search).
     * Return paged list of name elements.
     *
     * @param name name or uuid
     * @param orderBy order by field
     * @param isAsc sort order, ascending or descending
     * @param page page starting from 0, offset
     * @param pageSize page size, limit
     * @return paged list of name elements
     */
    @Operation(
            summary     = "Find valid names by name or uuid (search)",
            description = """
                          Find valid names by name or uuid (search).
                          Return paged array of name elements.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return paged array of name elements.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = ResponsePageNameElements.class))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            value = "/{name}",
            produces = {"application/json"})
    public ResponsePageNameElements readNames(
            @Parameter(in = ParameterIn.PATH,  description = "search by name or uuid") @PathVariable("name") String name,
            @Parameter(in = ParameterIn.QUERY, description = "order by field") @RequestParam(required = false) FieldName orderBy,
            @Parameter(in = ParameterIn.QUERY, description = "sort order, ascending or descending") @RequestParam(required = false) Boolean isAsc,
            @Parameter(in = ParameterIn.QUERY, description = "page starting from 0, offset") @RequestParam(required = false, defaultValue = DEFAULT_PAGE) Integer page,
            @Parameter(in = ParameterIn.QUERY, description = "page size, limit") @RequestParam(required = false, defaultValue = DEFAULT_PAGE_SIZE) Integer pageSize);

    /**
     * Find valid names by system structure mnemonic path (search).
     * Return paged list of name elements.
     *
     * @param mnemonicPath mnemonic path
     * @param orderBy order by field
     * @param isAsc sort order, ascending or descending
     * @param page page starting from 0, offset
     * @param pageSize page size, limit
     * @return paged list of name elements
     */
    @Operation(
            summary     = "Find valid names by system structure mnemonic path (search)",
            description = """
                          Find valid names by system structure mnemonic path (search).
                          Return paged array of name elements.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return paged array of name elements.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = ResponsePageNameElements.class))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            value = "/systemStructure/{mnemonicPath}",
            produces = {"application/json"})
    public ResponsePageNameElements readNamesSystemStructure(
            @Parameter(in = ParameterIn.PATH,  description = "search by system structure mnemonic path") @PathVariable("mnemonicPath") String mnemonicPath,
            @Parameter(in = ParameterIn.QUERY, description = "order by field") @RequestParam(required = false) FieldName orderBy,
            @Parameter(in = ParameterIn.QUERY, description = "sort order, ascending or descending") @RequestParam(required = false) Boolean isAsc,
            @Parameter(in = ParameterIn.QUERY, description = "page starting from 0, offset") @RequestParam(required = false, defaultValue = DEFAULT_PAGE) Integer page,
            @Parameter(in = ParameterIn.QUERY, description = "page size, limit") @RequestParam(required = false, defaultValue = DEFAULT_PAGE_SIZE) Integer pageSize);

    /**
     * Find valid names by device structure mnemonic path (search).
     * Return paged list of name elements.
     *
     * @param mnemonicPath mnemonic path
     * @param orderBy order by field
     * @param isAsc sort order, ascending or descending
     * @param page page starting from 0, offset
     * @param pageSize page size, limit
     * @return paged list of name elements
     */
    @Operation(
            summary     = "Find valid names by device structure mnemonic path (search)",
            description = """
                          Find valid names by device structure mnemonic path (search).
                          Return paged array of name elements.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return paged array of name elements.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = ResponsePageNameElements.class))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            value = "/deviceStructure/{mnemonicPath}",
            produces = {"application/json"})
    public ResponsePageNameElements readNamesDeviceStructure(
            @Parameter(in = ParameterIn.PATH,  description = "search by device structure mnemonic path") @PathVariable("mnemonicPath") String mnemonicPath,
            @Parameter(in = ParameterIn.QUERY, description = "order by field") @RequestParam(required = false) FieldName orderBy,
            @Parameter(in = ParameterIn.QUERY, description = "sort order, ascending or descending") @RequestParam(required = false) Boolean isAsc,
            @Parameter(in = ParameterIn.QUERY, description = "page starting from 0, offset") @RequestParam(required = false, defaultValue = DEFAULT_PAGE) Integer page,
            @Parameter(in = ParameterIn.QUERY, description = "page size, limit") @RequestParam(required = false, defaultValue = DEFAULT_PAGE_SIZE) Integer pageSize);

    /**
     * Find history for name by uuid (exact match).
     * Return paged list of name elements.
     *
     * @param uuid uuid
     * @param orderBy order by field
     * @param isAsc sort order, ascending or descending
     * @param page page starting from 0, offset
     * @param pageSize page size, limit
     * @return paged list of name elements
     */
    @Operation(
            summary     = "Find history for name by uuid (exact match)",
            description = """
                          Find history for name by uuid (exact match).
                          Return paged array of name elements.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return paged array of name elements.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = ResponsePageNameElements.class))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            value = "/history/{uuid}",
            produces = {"application/json"})
    public ResponsePageNameElements readNamesHistory(
            @Parameter(in = ParameterIn.PATH,  description = "search by uuid") @PathVariable("uuid") String uuid,
            @Parameter(in = ParameterIn.QUERY, description = "order by field") @RequestParam(required = false, defaultValue = DEFAULT_SORT_FIELD_WHEN) FieldName orderBy,
            @Parameter(in = ParameterIn.QUERY, description = "sort order, ascending or descending") @RequestParam(required = false, defaultValue = DEFAULT_SORT_ORDER_ASC) Boolean isAsc,
            @Parameter(in = ParameterIn.QUERY, description = "page starting from 0, offset") @RequestParam(required = false, defaultValue = DEFAULT_PAGE) Integer page,
            @Parameter(in = ParameterIn.QUERY, description = "page size, limit") @RequestParam(required = false, defaultValue = DEFAULT_PAGE_SIZE) Integer pageSize);

    @Operation(
            summary     = "Find valid legacy names (search)",
            description = """
                          Find valid legacy names (search).
                          Return paged array of name elements.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return paged array of name elements.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = ResponsePageNameElements.class))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            value = "/legacy",
            produces = {"application/json"})
    public ResponsePageNameElements readNamesLegacy(
            @Parameter(in = ParameterIn.QUERY, description = "search by name") @RequestParam(required = false) String name,
            @Parameter(in = ParameterIn.QUERY, description = "order by field") @RequestParam(required = false, defaultValue = DEFAULT_SORT_FIELD_WHEN) FieldName orderBy,
            @Parameter(in = ParameterIn.QUERY, description = "sort order, ascending or descending") @RequestParam(required = false, defaultValue = DEFAULT_SORT_ORDER_ASC) Boolean isAsc,
            @Parameter(in = ParameterIn.QUERY, description = "page starting from 0, offset") @RequestParam(required = false, defaultValue = DEFAULT_PAGE) Integer page,
            @Parameter(in = ParameterIn.QUERY, description = "page size, limit") @RequestParam(required = false, defaultValue = DEFAULT_PAGE_SIZE) Integer pageSize);

    // ----------------------------------------------------------------------------------------------------

    /**
     * Return name equivalence for name.
     *
     * @param name name to get name equivalence for
     * @return name equivalence
     */
    @Operation(
            summary     = "Return name equivalence for name",
            description = "Return name equivalence for name."
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return name equivalence.",
                    content = @Content(
                            mediaType = "text/plain",
                            schema = @Schema(implementation = String.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))

    })
    @GetMapping(
            value = "/equivalence/{name}")
    public String equivalenceName(
             @Parameter(in = ParameterIn.PATH, description = "name to get name equivalence for") @PathVariable("name") String name);

    /**
     * Return if name exists (exact match).
     *
     * <p>
     * Response is true if name exists, false otherwise.
     * Message and details are available if no response is available.
     * </p>
     *
     * @param name name to check if it exists
     * @return if name exists
     */
    @Operation(
            summary     = "Return if name exists (exact match)",
            description = """
                          Return if name exists (exact match).
                          Response is true if name exists, false otherwise.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = """
                                   OK. Response is true if name exists, false otherwise.
                                   Message and details are available if no response is available.
                                   """,
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = ResponseBoolean.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            value = "/exists/{name}",
            produces = {"application/json"})
    public ResponseEntity<ResponseBoolean> existsName(
             @Parameter(in = ParameterIn.PATH, description = "name to check if it exists") @PathVariable("name") String name);

    /**
     * Return if name is legacy name (exact match).
     * A name is considered legacy name if one or more of its parents is deleted.
     *
     * <p>
     * Response is true if name is legacy name, false otherwise.
     * Message and details are available if no response is available.
     * </p>
     *
     * @param name name to check if it is legacy name
     * @return if name is legacy name
     */
    @Operation(
            summary     = "Return if name is legacy name (exact match)",
            description = """
                          Return if name is legacy name (exact match).
                          A name is considered legacy name if one or more of its parents is deleted.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = """
                                   OK. Response is true if name is legacy name, false otherwise.
                                   Message and details are available if no response is available.
                                   """,
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = ResponseBoolean.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            value = "/isLegacy/{name}",
            produces = {"application/json"})
    public ResponseEntity<ResponseBoolean> isLegacyName(
            @Parameter(in = ParameterIn.PATH, description = "name to check if it is legacy name") @PathVariable("name") String name);

    /**
     * Return if name is valid to create (exact match).
     * Method answers question 'would it be ok to create given name?'.
     *
     * <p>
     * Response is true if name is valid to create, false otherwise.
     * Message and details are available if no response is available.
     * </p>
     *
     * @param name name to check if it is valid to create
     * @return if name is valid to create
     */
    @Operation(
            summary     = "Return if name is valid to create (exact match)",
            description = """
                          Return if name is valid to create (exact match).
                          Method answers question 'would it be ok to create given name?'.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = """
                                   OK. Response is true if name is valid to create, false otherwise.
                                   Message and details are available if no response is available.
                                   """,
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = ResponseBoolean.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            value = "/isValidToCreate/{name}",
            produces = {"application/json"})
    public ResponseEntity<ResponseBoolean> isValidToCreateName(
            @Parameter(in = ParameterIn.PATH, description = "name to check if it is valid to create") @PathVariable("name") String name);

    // ----------------------------------------------------------------------------------------------------

    /**
     * Return if names are valid to create by list of name element commands for create.
     * If names are valid to create, successful create of names can be expected.
     *
     * <p>
     * Response is true if all name element commands validated ok, false otherwise, responses contain array
     * with result for each name element command. Message and details are available if no response is available.
     * </p>
     *
     * @param nameElementCommands list of name element commands for create
     * @return if list of names is valid to create
     */
    @Hidden
    @Operation(
            summary     = "Return if names are valid to create by list of name element commands for create",
            description = """
                          Return if names are valid to create by list of name element commands for create.
                          If names are valid to create, successful create of names can be expected.

                          Required attributes:
                          - parentSystemStructure (uuid)
                          - description
                          - comment

                          Optional attributes:
                          - parentDeviceStructure (uuid)
                          - index

                          Uuid is created by Naming.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = """
                                   OK. Response is true if all name element commands validated ok, false otherwise.
                                   Responses contain array with result for each name element command. Message and details are available if no response is available.
                                   """,
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = ResponseBooleanList.class))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            value = "/validateCreate",
            produces = {"application/json"},
            consumes = {"application/json"})
    public ResponseEntity<ResponseBooleanList> validateNamesCreate(
            @Parameter(
                    in = ParameterIn.DEFAULT,
                    description = "array of name element commands",
                    required = true,
                    content = @Content(
                            mediaType = "application/json",
                            array = @ArraySchema(
                                    schema = @Schema(
                                            implementation = NameElementCommandCreate.class,
                                            requiredProperties = {"parentSystemStructure","parentDeviceStructure","index","description","comment"}))))
            @RequestBody List<NameElementCommandCreate> nameElementCommands);

    /**
     * Return if names are valid to update by list of name element commands for update.
     * If names are valid to update, successful update of names can be expected.
     *
     * <p>
     * Response is true if all name element commands validated ok, false otherwise, responses contain array
     * with result for each name element command. Message and details are available if no response is available.
     * </p>
     *
     * @param nameElementCommands list of name element commands for update
     * @return if list of name elements is valid to update
     */
    @Hidden
    @Operation(
            summary     = "Return if names are valid to update by list of name elements for update",
            description = """
                          Return if names are valid to update by list of name elements for update.
                          If names are valid to update, successful update of names can be expected.

                          Required attributes:
                          - uuid
                          - parentSystemStructure (uuid)
                          - description
                          - comment

                          Optional attributes:
                          - parentDeviceStructure (uuid)
                          - index
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = """
                                   OK. Response is true if all name element commands validated ok, false otherwise.
                                   Responses contain array with result for each name element command. Message and details are available if no response is available.
                                   """,
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = ResponseBooleanList.class))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            value = "/validateUpdate",
            produces = {"application/json"},
            consumes = {"application/json"})
    public ResponseEntity<ResponseBooleanList> validateNamesUpdate(
            @Parameter(
                    in = ParameterIn.DEFAULT,
                    description = "array of name element commands",
                    required = true,
                    content = @Content(
                            mediaType = "application/json",
                            array = @ArraySchema(
                                    schema = @Schema(
                                            implementation = NameElementCommandUpdate.class,
                                            requiredProperties = {"uuid","parentSystemStructure","description","comment"}))))
            @RequestBody List<NameElementCommandUpdate> nameElementCommands);

    /**
     * Return if names are valid to delete by list of name element commands for confirm.
     * If names are valid to delete, successful delete of names can be expected.
     *
     * <p>
     * Response is true if all name element commands validated ok, false otherwise, responses contain array
     * with result for each name element command. Message and details are available if no response is available.
     * </p>
     *
     * @param nameElementCommands list of name element commands for confirm
     * @return if list of name elements is valid to delete
     */
    @Hidden
    @Operation(
            summary     = "Return if names are valid to delete by list of name element commands for confirm",
            description = """
                          Return if names are valid to delete by list of name element commands for confirm.
                          If names are valid to delete, successful delete of names can be expected.

                          Required attributes:
                          - uuid
                          - comment

                          Other attributes are not considered.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = """
                                   OK. Response is true if all name element commands validated ok, false otherwise.
                                   Responses contain array with result for each name element command. Message and details are available if no response is available.
                                   """,
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = ResponseBooleanList.class))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            value = "/validateDelete",
            produces = {"application/json"},
            consumes = {"application/json"})
    public ResponseEntity<ResponseBooleanList> validateNamesDelete(
            @Parameter(
                    in = ParameterIn.DEFAULT,
                    description = "array of name element commands",
                    required = true,
                    content = @Content(
                            mediaType = "application/json",
                            array = @ArraySchema(
                                    schema = @Schema(
                                            implementation = NameElementCommandConfirm.class,
                                            requiredProperties = {"uuid","comment"}))))
            @RequestBody List<NameElementCommandConfirm> nameElementCommands);

    // ----------------------------------------------------------------------------------------------------

    /**
     * Update names by list of name element commands for update.
     * Return list of name elements for updated names.
     *
     * @param nameElementCommands list of name element commands for update
     * @return list of name elements for updated names
     */
    @Operation(
            summary     = "Update names by array of name element commands for update",
            description = """
                          Update names by array of name element commands for update.
                          Return array of name elements for updated names.

                          Required attributes:
                          - uuid
                          - parentSystemStructure (uuid)
                          - description
                          - comment

                          Optional attributes:
                          - parentDeviceStructure (uuid)
                          - index
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return array of name elements for updated names.",
                    content = @Content(
                            mediaType = "application/json",
                            array = @ArraySchema(schema = @Schema(implementation = NameElement.class)))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "404",
                    description  = "Not Found. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "409",
                    description  = "Conflict. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @PutMapping(
            produces = {"application/json"},
            consumes = {"application/json"})
    public List<NameElement> updateNames(
            @Parameter(
                    in = ParameterIn.DEFAULT,
                    description = "array of name element commands",
                    required = true,
                    content = @Content(
                            mediaType = "application/json",
                            array = @ArraySchema(
                                    schema = @Schema(
                                            implementation = NameElementCommandUpdate.class,
                                            requiredProperties = {"uuid","parentSystemStructure","description","comment"}))))
            @RequestBody List<NameElementCommandUpdate> nameElementCommands);

    /**
     * Update names by upload Excel file.
     * Return Excel file with name elements for updated names.
     *
     * @param file file
     * @return Excel file
     */
    @Operation(
            summary     = "Update names by upload Excel file",
            description = """
                          Update names by upload Excel file.
                          Return Excel file with name elements for updated names.

                          Expected columns as in Excel template for names.

                          Required attributes:
                          - uuid
                          - parentSystemStructure (uuid)
                          - description
                          - comment

                          Optional attributes:
                          - parentDeviceStructure (uuid)
                          - index
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return Excel file with name elements for updated names.",
                    content = @Content(mediaType = ExcelUtil.MIME_TYPE_EXCEL)),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "404",
                    description  = "Not Found. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "409",
                    description  = "Conflict. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @PutMapping(
            value = "/upload",
               produces = {"application/vnd.ms-excel"},
            consumes = {"multipart/form-data"})
    public ResponseEntity<Resource> updateNames(
            @Parameter(
                    in = ParameterIn.DEFAULT,
                    description = "Excel file with name element commands",
                    required = true,
                    content = @Content(mediaType = ExcelUtil.MIME_TYPE_OPENXML_SPREADSHEET))
            @RequestParam("file") MultipartFile file);

    // ----------------------------------------------------------------------------------------------------

    /**
     * Delete names by list of name element commands for confirm.
     * Return list of name elements for deleted names.
     *
     * @param nameElementCommands list of name element commands for confirm
     * @return list of name elements for deleted names
     */
    @Operation(
            summary     = "Delete names by array of name element commands for confirm",
            description = """
                          Delete names by array of name element commands for confirm.
                          Return array of name elements for deleted names.

                          Required attributes:
                          - uuid
                          - comment

                          Other attributes are not considered.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return array of name elements for deleted names.",
                    content = @Content(
                            mediaType = "application/json",
                            array = @ArraySchema(schema = @Schema(implementation = NameElement.class)))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "404",
                    description  = "Not Found. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @DeleteMapping(
            produces = {"application/json"},
            consumes = {"application/json"})
    public List<NameElement> deleteNames(
            @Parameter(
                    in = ParameterIn.DEFAULT,
                    description = "array of name element commands",
                    required = true,
                    content = @Content(
                            mediaType = "application/json",
                            array = @ArraySchema(
                                    schema = @Schema(
                                            implementation = NameElementCommandConfirm.class,
                                            requiredProperties = {"uuid","comment"}))))
            @RequestBody List<NameElementCommandConfirm> nameElementCommands);

    /**
     * Delete names by upload Excel file.
     * Return Excel file with name elements for deleted names.
     *
     * @param file file
     * @return Excel file
     */
    @Operation(
            summary     = "Delete names by upload Excel file",
            description = """
                          Delete names by upload Excel file.
                          Return Excel file with name elements for deleted names.

                          Expected columns as in Excel template for names.

                          Required attributes:
                          - uuid
                          - comment

                          Other attributes are not considered.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return Excel file with name elements for deleted names.",
                    content = @Content(mediaType = ExcelUtil.MIME_TYPE_EXCEL)),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "404",
                    description  = "Not Found. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @DeleteMapping(
            value = "/upload",
            produces = {"application/vnd.ms-excel"},
            consumes = {"multipart/form-data"})
    public ResponseEntity<Resource> deleteNames(
            @Parameter(
                    in = ParameterIn.DEFAULT,
                    description = "Excel file with name element commands",
                    required = true,
                    content = @Content(mediaType = ExcelUtil.MIME_TYPE_OPENXML_SPREADSHEET))
            @RequestParam("file") MultipartFile file);

}
